
// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// ====================================================================================
// 
//  __        __        __
//  \ \       /  \       / /   	[File name   ] video_top.v
//   \ \  / /\ \  / /     		[Description ] GOWIN ISP demo
//      \ \/ /  \ \/ /     		[Timestamp   ] 
//       \  /       \  /        [version     ] 1.0.0
//        \/        \/
//
// ==============0ooo===================================================0ooo===========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:    |  Author    | Mod. Date    | Changes Made:
// ----------------------------------------------------------------------------------
// V1.0    | HKGOWIN     | 05/03/2021  | Initial version 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo===========

`include "vfb_defines.vh"

module video_top
(
    input          I_clk           , //50Mhz
    input          I_rst_n         ,
    output [3:0]   O_led           ,
    input [3:0]    I_isp_mode      ,   
	
    input          I_CSI_CKP       , 
    input          I_CSI_CKN       ,                          
    input          I_CSI_D0P       , 
    input          I_CSI_D0N       ,
    input          I_CSI_D1P       , 
    input          I_CSI_D1N       ,
    inout  [1:0]   I_CSI_LPCLK     ,
    inout  [1:0]   I_CSI_LP0       ,
    inout  [1:0]   I_CSI_LP1       ,
    output         O_csi_resetb    ,
    output         O_csi_pwdn      ,//O_csi_clk
    output         O_csi_scl       ,
    inout          IO_csi_sda      ,

    output         LCD_MODE        ,
    output         LCD_DE          ,
    output         LCD_VS          ,
    output         LCD_HS          ,
    output [7:2]   LCD_B           ,
    output [7:2]   LCD_G           ,
    output [7:2]   LCD_R           ,
    output         LCD_DCLK        ,
    output         LCD_RL          ,
    output         LCD_UD          ,
    
	output			MCU_UART_TX,
	input			MCU_UART_RX,
	
	
    output [13:0]  O_ddr_addr      ,
    output [2:0]   O_ddr_ba        ,
    output         O_ddr_cs_n      ,
    output         O_ddr_ras_n     ,
    output         O_ddr_cas_n     ,
    output         O_ddr_we_n      ,
    output         O_ddr_clk       ,
    output         O_ddr_clk_n     ,
    output         O_ddr_cke       ,
    output         O_ddr_odt       ,
    output         O_ddr_reset_n   ,
    output [1:0]   O_ddr_dqm       ,
    inout  [15:0]  IO_ddr_dq       ,
    inout  [1:0]   IO_ddr_dqs      ,
    inout  [1:0]   IO_ddr_dqs_n    
);
//===================================================================
// --- Parameter ---
//===================================================================
// --- SRAM parameters ---
	parameter IMAGE_SIZE          = 28'h080_0000;//--8MB,byte address  //frame base address
	parameter BURST_WRITE_LENGTH  = 1024;  //bytes
	parameter BURST_READ_LENGTH   = 1024;  //bytes
	parameter ADDR_WIDTH          = 28;    //存储单元是byte，总容量=2^27*16bit = 2Gbit,增加1位rank地址，{rank[0],bank[2:0],row[13:0],cloumn[9:0]}
	parameter DATA_WIDTH          = 128;   //与生成DDR3IP有关，此ddr3 2Gbit, x16， 时钟比例1:4 ，则固定128bit
	parameter DQ_WIDTH            = 16;
	parameter VIDEO_WIDTH          = `DEF_VIDEO_WIDTH;  
	
	
//===================================================================
// --- Signals ---
//===================================================================
	wire        rst_n1;
	wire        rst_n2;
	wire        ddr_rstn;

//===================================================================
	reg  [31:0] run_cnt;
	wire        running;

	wire        config_finished;

//===================================================================
	wire        pixel_clk;
	wire        pixel_clkx2;

	wire        ddr3_clk;
    wire        ddr3_clk_lock;					 
//===================================================================
	wire        tp0_vs_in  ;
	wire        tp0_hs_in  ;
	wire        tp0_de_in ;
	wire [ 7:0] tp0_data_r;
	wire [ 7:0] tp0_data_g;
	wire [ 7:0] tp0_data_b;

	reg         vs_r;
	reg  [9:0]  cnt_vs;

//===================================================================
// --- MIPI CSI ---
	wire        csi_clk ;
	wire        csi_vs  ;
	wire        csi_de  ;
	wire [7:0]  csi_data/*synthesis syn_keep=1*/;

//===================================================================
// --- bayer2RGB ---
	wire        b2rgb_vs;  
	wire        b2rgb_hs;  
	wire        b2rgb_de;  
	wire [7:0]  b2rgb_data_R;
	wire [7:0]  b2rgb_data_G;
	wire [7:0]  b2rgb_data_B;

//===================================================================
// --- frame clip ---
	wire         clip_vs     ;//Negative
	wire         clip_de     ;
//	wire [7:0]   clip_data   ;
	wire [23:0]   clip_data   ;

// --- frame scale ---
    wire         sclout_vs;
    wire         sclout_de;
    wire [23:0]  sclout_data;
//===================================================================
// --- frame buffer in ---
	wire        			ch0_vfb_vs_in  ;
	wire        			ch0_vfb_de_in  ;
	wire [VIDEO_WIDTH-1:0] 	ch0_vfb_data_in; 

	wire        			ch1_vfb_vs_in  ;
	wire        			ch1_vfb_de_in  ;
	wire [VIDEO_WIDTH-1:0]	ch1_vfb_data_in; 

	wire [VIDEO_WIDTH-1:0] 	ch0_data_out;
	wire [VIDEO_WIDTH-1:0] 	ch1_data_out;


//===================================================================
// ---ddr3 ---
	wire        dma_clk  ;  

//===================================================================
// --- memory interface ---
	wire                   cmd_ready          ;
	wire[2:0]              cmd                ;
	wire                   cmd_en             ;
	wire[5:0]              app_burst_number   ;
	wire[ADDR_WIDTH-1:0]   addr               ;
	wire                   wr_data_rdy        ;
	wire                   wr_data_en         ;//
	wire                   wr_data_end        ;//
	wire[DATA_WIDTH-1:0]   wr_data            ;   
	wire[DATA_WIDTH/8-1:0] wr_data_mask       ;   
	wire                   rd_data_valid      ;  
	wire                   rd_data_end        ;//unused 
	wire[DATA_WIDTH-1:0]   rd_data            ;   
	wire                   init_calib_complete;

//===================================================================
// --- syn_code ---
	wire                   syn_off0_re/*synthesis syn_keep=1*/;  // ofifo read enable signal
	wire                   syn_off0_de/*synthesis syn_keep=1*/;
	wire                   syn_off0_vs/*synthesis syn_keep=1*/;
	wire                   syn_off0_hs/*synthesis syn_keep=1*/;
	reg  [4:0]             syn_off0_de_d;
	reg  [4:0]             syn_off0_vs_d;
	reg  [4:0]             syn_off0_hs_d;
				  
	wire                   off0_syn_de  /*synthesis syn_keep=1*/;
	wire [VIDEO_WIDTH-1:0] off0_syn_data/*synthesis syn_keep=1*/;

	wire                   syn_off1_re;  // ofifo read enable signal
	wire                   syn_off1_de;  // ofifo read enable signal
	wire                   syn_off1_vs;
	wire                   syn_off1_hs;
	reg  [4:0]             syn_off1_de_d;
	reg  [4:0]             syn_off1_vs_d;
	reg  [4:0]             syn_off1_hs_d;

	wire                   off1_syn_de  ;
	wire [VIDEO_WIDTH-1:0] off1_syn_data;


	wire				   syn_off0_frame_sync;
	wire				   syn_off0_frame_end;

//===================================================================
// --- AHB ---
	wire [31:0] AHB1HRDATA;      // Data from slave to master
	wire        AHB1HREADYOUT;   // Slave ready signal
	wire [1:0]  AHB1HRESP;       // Slave response signal  
	wire [1:0]  AHB1HTRANS;      // Transfer type
	wire [2:0]  AHB1HBURST;      // Burst type
	wire [3:0]  AHB1HPROT;       // Transfer protection bits
	wire [2:0]  AHB1HSIZE;       // Transfer size
	wire        AHB1HWRITE;      // Transfer direction
	wire        AHB1HMASTLOCK;   // Transfer is a locked transfer
	wire        AHB1HREADYMUX;   // Ready mux signal
	wire [3:0]  AHB1HMASTER;     // Transfer is a locked transfer
	wire [31:0] AHB1HADDR;       // Transfer address
	wire [31:0] AHB1HWDATA;      // Data from master to slave
	wire        AHB1HSEL;        // Select
	wire        AHB1HCLK;        // Bus clock signal
	wire        AHB1HRESET;      // Bus reset signal


//===================================================================
// --- isp ---
	wire isp_clk;
	wire isp_vs_out;
	wire isp_de_out;
	wire [7:0] isp_data_R;
	wire [7:0] isp_data_G;
	wire [7:0] isp_data_B;
	
	wire isp_rd_rdy;
	wire isp_reg_wr_en;
	wire [15:0] isp_reg_addr;
	wire [31:0] isp_reg_wr_data;
	wire isp_reg_rd_en;
	wire [31:0] isp_reg_rd_data;
	wire [3:0] isp_disp_mode;
    wire isp_image_mask;

//===================================================================
// --- sensor reg ---
	wire update_valid;
	wire cam_awb_en;
	wire [15:0] cam_awb_gain_r; 
	wire [15:0] cam_awb_gain_g;
	wire [15:0] cam_awb_gain_b; 
	wire cam_agc_en;
	wire [15:0] cam_agc_gain; 
	wire        cam_aec_en; 
	wire [19:0] cam_aec_exposure; 

//===================================================================
// --- Module
//===================================================================
// --- pixel clock ---
    Gowin_rPLL pixel_pll(
        .clkout(pixel_clk), //output clkout
        .clkin (I_clk) //input clkin
    );
	DDR3_rPLL ddr3_pll(
        .clkout(ddr3_clk), //output clkout
        .lock(ddr3_clk_lock), //output lock
        .clkin(I_clk) //input clkin
    );				   
//===================================================================
// --- LED灯测试 ---
	always @(posedge I_clk or negedge I_rst_n)begin	//I_clk
		if(!I_rst_n)
			run_cnt <= 32'd0;
		else if(run_cnt >= 32'd50_000_000)
			run_cnt <= 32'd0;
		else
			run_cnt <= run_cnt + 1'b1;
	end

	assign  running = (run_cnt < 32'd25_000_000) ? 1'b1 : 1'b0;

	assign  O_led[0] = ~init_calib_complete;
	assign  O_led[1] = running;
	assign  O_led[2] = ~config_finished;
	assign  O_led[3] = AHB1HRESET;

//===================================================================
// --- OV5647 Camera initial ---
	OV5647_Controller OV5647_Controller_inst(
		.clk            (I_clk       ),   // 50Mhz clock signal
		.rstn           (I_rst_n      ),
		.config_finished(config_finished),   // Flag to indicate that the configuration is finished
		.sioc           (O_csi_scl   ),   // SCCB interface - clock signal
		.siod           (IO_csi_sda  ),   // SCCB interface - data signal
		.resetb         (O_csi_resetb),   // RESET signal for OV5647
		.pwdn           (O_csi_pwdn  ),    // PWDN signal for OV5647
		
		.reg_update_valid 	(update_valid),
		.reg_awb_en			(cam_awb_en),
		.reg_awb_gain_r		(cam_awb_gain_r),
		.reg_awb_gain_g		(cam_awb_gain_g),
		.reg_awb_gain_b		(cam_awb_gain_b),
		.reg_agc_en			(cam_agc_en),
		.reg_agc_gain		(cam_agc_gain),
		.reg_aec_en		    (cam_aec_en),
		.reg_aec_exposure	(cam_aec_exposure)
	);


//===================================================================
// --- CSI 2 RAW DATA ---
	CSI2RAW8 CSI2RAW8_inst(
		 .rstn      (I_rst_n     ) 
		,.DCK_P     (I_CSI_CKP  )                                
		,.CH0_P     (I_CSI_D0P  )
		,.CH1_P     (I_CSI_D1P  )
		,.DCK_N     (I_CSI_CKN  )
		,.CH0_N     (I_CSI_D0N  )
		,.CH1_N     (I_CSI_D1N  )
		,.LPCLK     (I_CSI_LPCLK)
		,.LP0       (I_CSI_LP0  )
		,.LP1       (I_CSI_LP1  )
		,.pixclk    (csi_clk    ) 
		,.pixdata   (csi_data   )  
		,.vs        (csi_vs     ) 
		,.de        (csi_de     )
	);

/*
//===================================================================
// --- test pattern ---
	testpattern testpattern_inst(
		.I_pxl_clk   (pixel_clk          ),//pixel clock
		.I_rst_n     (I_rst_n            ),//low active 
		.I_mode      ({1'b0,cnt_vs[9:7]} ),//data select
		.I_single_r  (8'd0               ),
		.I_single_g  (8'd255             ),
		.I_single_b  (8'd0               ),//800x480   //800x600     //1280x720    
		.I_h_total   (16'd1056           ),// 16'd1056 // 16'd1056   // 16'd1650  
		.I_h_sync    (16'd20             ),// 16'd20   // 16'd128    // 16'd40    
		.I_h_bporch  (16'd26             ),// 16'd26   // 16'd88     // 16'd220   
		.I_h_res     (16'd800            ),// 16'd800  // 16'd800    // 16'd1280  
		.I_v_total   (16'd525            ),// 16'd525  // 16'd628    // 16'd750    
		.I_v_sync    (16'd4              ),// 16'd4    // 16'd4      // 16'd5     
		.I_v_bporch  (16'd19             ),// 16'd19   // 16'd23     // 16'd20    
		.I_v_res     (16'd480            ),// 16'd480  // 16'd600    // 16'd720    
		.I_hs_pol    (1'b0               ),//0,负极性;1,正极性
		.I_vs_pol    (1'b0               ),//0,负极性;1,正极性
		.O_de        (tp0_de_in          ),   
		.O_hs        (tp0_hs_in          ),
		.O_vs        (tp0_vs_in          ),
		.O_data_r    (tp0_data_r         ),   
		.O_data_g    (tp0_data_g         ),
		.O_data_b    (tp0_data_b         )
	);

//------------------------------------
	always@(posedge pixel_clk)//clkout )
	begin
		vs_r<=tp0_vs_in;
	end

	always@(posedge pixel_clk or negedge I_rst_n)//clkout )
	begin
		if(!I_rst_n)
			cnt_vs<=0;
		else if(vs_r && !tp0_vs_in) //vs24 falling edge
			cnt_vs<=cnt_vs+1;
	end 
*/

//===================================================================
// --- isp module ---
	isp_top  #(
		.DATA_WIDTH(8)
	)isp_inst(
		.clk(csi_clk), 
		.rstn(I_rst_n),

		.in_vs(csi_vs),
		.in_de(csi_de),
		.in_data(csi_data),
		
		.isp_rd_rdy(isp_rd_rdy),
		.isp_reg_wr_en(isp_reg_wr_en),
		.isp_reg_addr(isp_reg_addr),
		.isp_reg_wr_data(isp_reg_wr_data),
		.isp_reg_rd_en(isp_reg_rd_en),
		.isp_reg_rd_data(isp_reg_rd_data),
        .isp_disp_mode(I_isp_mode),
	
		.out_vs(isp_vs_out),
		.out_de(isp_de_out),
		.out_data_R(isp_data_R),
		.out_data_G(isp_data_G),
		.out_data_B(isp_data_B)
	);

//===================================================================
// --- from 1920x1080 scaler to 960x540 ---
 Scaler_Top Scaler_Top_inst(
     .I_reset        (~I_rst_n          ), 
     .I_sysclk       (csi_clk           ), 
     .I_vin_clk      (csi_clk           ), 
     .I_vin_vs_cpl   (~isp_vs_out           ), //positive
     .I_vin_de_cpl   (isp_de_out            ), 
     .I_vin_data0_cpl(isp_data_R        ), 
     .I_vin_data1_cpl(isp_data_G        ), 
     .I_vin_data2_cpl(isp_data_B        ), 
     .O_vout0_data   (sclout_data[23:16]), 
     .O_vout1_data   (sclout_data[15: 8]), 
     .O_vout2_data   (sclout_data[ 7: 0]), 
     .I_vout_clk     (csi_clk           ), 
     .O_vout_vs      (sclout_vs         ), //positive
     .O_vout_de      (sclout_de         )  
 );

	video_clip_sl #(
		.DATA_WIDTH(24)
	)video_clip_sl_inst(
		.rst_n             (I_rst_n      ),
		.pxclk             (csi_clk      ),
		.vsync_i           (~sclout_vs   ),//negetive
		.hsync_i           (1'b1         ),//negetive
		.data_en_i         (sclout_de    ),
		.data_i            (sclout_data  ),
		.begin_point_x_ofst(13'd40      ),
		.begin_point_y_ofst(13'd20      ),
		.resolution_x      (13'd800      ),
		.resolution_y      (13'd480      ),
		.vsync_o           (clip_vs      ),
		.hsync_o           (             ),
		.data_en_o         (clip_de      ),
		.data_o	           (clip_data    )      
	);

//===================================================================
// --- videl frame channel ---
 `ifdef VIDEO_WIDTH_16
	 assign ch0_vfb_clk_in  = csi_clk;//clkout     ; //I_adv7611_clk  
	 assign ch0_vfb_vs_in   = clip_vs;
	 assign ch0_vfb_de_in   = clip_de;
	 assign ch0_vfb_data_in = {clip_data[23:19],clip_data[15:10],clip_data[7:3]};
 `endif


//===================================================================
// --- SRAM 控制模块 ---
	vfb_top#(
		.IMAGE_SIZE        (IMAGE_SIZE        ),     
		.BURST_WRITE_LENGTH(BURST_WRITE_LENGTH),
		.BURST_READ_LENGTH (BURST_READ_LENGTH ),
		.ADDR_WIDTH        (ADDR_WIDTH        ),
		.DATA_WIDTH        (DATA_WIDTH        ),
		.DQ_WIDTH          (DQ_WIDTH          ),
		.VIDEO_WIDTH       (VIDEO_WIDTH       )
	)vfb_top_inst( 
		.I_rst_n		    (init_calib_complete ),//rst_n            ),
		.I_dma_clk		    (dma_clk          ),   //sram_clk         ),
		.I_wr_halt		    (0                 ), //1:halt,  0:no halt
		.I_rd_halt		    (0               ), //1:halt,  0:no halt
		// video data input           
		.I_vin0_clk		    (ch0_vfb_clk_in   ),
		.I_vin0_vs_n	    (ch0_vfb_vs_in    ),
		.I_vin0_de		    (ch0_vfb_de_in    ),
		.I_vin0_data	    (ch0_vfb_data_in  ),

		// video data output          
		.I_vout0_clk	    (pixel_clk          ),
		.I_vout0_vs_n	    (~syn_off0_vs    ),
		.I_vout0_de		    (syn_off0_re       ),
		.O_vout0_den	    (                 ),
		.O_vout0_data	    (off0_syn_data     ),

		// ddr write request
		.cmd_ready          (cmd_ready          ),
		.cmd                (cmd                ),//0:write;  1:read
		.cmd_en             (cmd_en             ),
		.app_burst_number   (app_burst_number   ),
		.addr               (addr               ),//[ADDR_WIDTH-1:0]
		.wr_data_rdy        (wr_data_rdy        ),
		.wr_data_en         (wr_data_en         ),//
		.wr_data_end        (wr_data_end        ),//
		.wr_data            (wr_data            ),//[DATA_WIDTH-1:0]
		.wr_data_mask       (wr_data_mask       ),
		.rd_data_valid      (rd_data_valid      ),
		.rd_data_end        (rd_data_end        ),//unused 
		.rd_data            (rd_data            ),//[DATA_WIDTH-1:0]
		.init_calib_complete(init_calib_complete)
	);  

//===================================================================
// --- DDR3 Interface ---
	DDR3_Memory_Interface_Top DDR3_Memory_Interface_Top_inst (
		.memory_clk         (ddr3_clk           ),       //input memory_clk
        .pll_lock           (ddr3_clk_lock      ),       //input pll_lock																
		.clk                (I_clk              ),
		.rst_n              (I_rst_n            ), //rst_n1
		.app_burst_number   (app_burst_number   ),
		.cmd_ready          (cmd_ready          ),
		.cmd                (cmd                ),
		.cmd_en             (cmd_en             ),
		.addr               (addr               ),
		.wr_data_rdy        (wr_data_rdy        ),
		.wr_data            (wr_data            ),
		.wr_data_en         (wr_data_en         ),
		.wr_data_end        (wr_data_end        ),
		.wr_data_mask       (wr_data_mask       ),
		.rd_data            (rd_data            ),
		.rd_data_valid      (rd_data_valid      ),
		.rd_data_end        (rd_data_end        ),
		.sr_req             (                   ),
		.ref_req            (                   ),
		.sr_ack             (                   ),
		.ref_ack            (                   ),
		.init_calib_complete(init_calib_complete),
		.clk_out            (dma_clk            ),
		.burst              (1'b1               ),
		// mem interface
		.ddr_rst            (                   ),
		.O_ddr_addr         (O_ddr_addr         ),
		.O_ddr_ba           (O_ddr_ba           ),
		.O_ddr_cs_n         (O_ddr_cs_n         ),
		.O_ddr_ras_n        (O_ddr_ras_n        ),
		.O_ddr_cas_n        (O_ddr_cas_n        ),
		.O_ddr_we_n         (O_ddr_we_n         ),
		.O_ddr_clk          (O_ddr_clk          ),
		.O_ddr_clk_n        (O_ddr_clk_n        ),
		.O_ddr_cke          (O_ddr_cke          ),
		.O_ddr_odt          (O_ddr_odt          ),
		.O_ddr_reset_n      (O_ddr_reset_n      ),
		.O_ddr_dqm          (O_ddr_dqm          ),
		.IO_ddr_dq          (IO_ddr_dq          ),
		.IO_ddr_dqs         (IO_ddr_dqs         ),
		.IO_ddr_dqs_n       (IO_ddr_dqs_n       )
	);

//===================================================================
// --- 输出同步时序模块 ---
	syn_gen syn_gen_inst0(                                   
		.I_pxl_clk   (pixel_clk       ), //33.3MHz   //40MHz      //74.25MHz  
		.I_rst_n     (I_rst_n         ), //800x480   //800x600    //1280x720      
		.I_h_total   (16'd1056        ), // 16'd1056 // 16'd1056  // 16'd1650   
		.I_h_sync    (16'd20          ), // 16'd20   // 16'd128   // 16'd40    
		.I_h_bporch  (16'd26          ), // 16'd26   // 16'd88    // 16'd220    
		.I_h_res     (16'd800         ), // 16'd800  // 16'd800   // 16'd1280   
		.I_v_total   (16'd525         ), // 16'd525  // 16'd628   // 16'd750     
		.I_v_sync    (16'd4           ), // 16'd4    // 16'd4     // 16'd5       
		.I_v_bporch  (16'd19          ), // 16'd19   // 16'd23    // 16'd20       
		.I_v_res     (16'd480         ), // 16'd480   // 16'd720     
		.I_rd_hres   (16'd800         ),
		.I_rd_vres   (16'd480         ),
		.I_hs_pol    (1'b1            ),//HS polarity , 0:负极性，1：正极性
		.I_vs_pol    (1'b1            ),//VS polarity , 0:负极性，1：正极性
		.O_rden      (syn_off0_re     ),
		.O_de        (syn_off0_de     ),   
		.O_hs        (syn_off0_hs     ),
		.O_vs        (syn_off0_vs     )
	);


	always@(posedge pixel_clk or negedge I_rst_n) begin
		if(!I_rst_n) begin                          
			syn_off0_de_d  <= {5{1'b0}};
			syn_off0_vs_d  <= {5{1'b1}}; 
			syn_off0_hs_d  <= {5{1'b1}}; 
		end
		else begin                          
			syn_off0_de_d  <= {syn_off0_de_d[5-2:0],syn_off0_de}; 
			syn_off0_vs_d  <= {syn_off0_vs_d[5-2:0],syn_off0_vs}; 
			syn_off0_hs_d  <= {syn_off0_vs_d[5-2:0],syn_off0_hs}; 
		end
	end

//===================================================================
// --- LCD Display ---
    assign LCD_DCLK = pixel_clk;    // 26.4M, 33.3M, 46.8M
    assign LCD_MODE = 1'b1;    // 0 --- SYNC MODE;     1 --- DE MODE
    assign LCD_UD = 1'b1;        // 0 --- Up to dowm;     1 --- Down to up
    assign LCD_RL = 1'b0;        // 0 --- Right to left; 1 --- Left to right
    assign LCD_DE = syn_off0_de_d[0];        // SYNC MODE, DE must be grounded.    
    assign LCD_HS = syn_off0_hs_d[0];
    assign LCD_VS = syn_off0_vs_d[0];
    assign LCD_R = {off0_syn_data[15:11],1'b0};
    assign LCD_G = off0_syn_data[10:5]        ;
    assign LCD_B = {off0_syn_data[4:0],1'b0}  ; 
	
//===================================================================
// --- MCU M1 ---	
	Gowin_EMPU_M1_Top Gowin_EMPU_M1_Top_inst(
		.LOCKUP         (),
	.GPIOIN			({15'b0, isp_rd_rdy}),
		.GPIOOUT		(),
		.GPIOOUTEN		(),
		.UART0RXD       (MCU_UART_RX),
		.UART0TXD       (MCU_UART_TX),  
		.AHB1HRDATA		(AHB1HRDATA),       // Data from slave to master
		.AHB1HREADYOUT	(AHB1HREADYOUT), // Slave ready signal, from slave, 1'b1
		.AHB1HRESP		(AHB1HRESP),         // Slave response signal  
		.AHB1HTRANS		(AHB1HTRANS),       // Transfer type
		.AHB1HBURST		(AHB1HBURST),       // Burst type
		.AHB1HPROT		(AHB1HPROT),         // Transfer protection bits
		.AHB1HSIZE		(AHB1HSIZE),         // Transfer size
		.AHB1HWRITE		(AHB1HWRITE),       // Transfer direction
		.AHB1HREADYMUX	(AHB1HREADYMUX), // Ready mux
		.AHB1HMASTLOCK	(AHB1HMASTLOCK), // Transfer is a locked transfer
		.AHB1HMASTER	(AHB1HMASTER),     // Transfer is a locked transfer
		.AHB1HADDR		(AHB1HADDR),         // Transfer address
		.AHB1HWDATA		(AHB1HWDATA),       // Data from master to slave
		.AHB1HSEL		(AHB1HSEL),           // Select
		.AHB1HCLK		(AHB1HCLK),           // Clock
		.AHB1HRESET		(AHB1HRESET),       // Reset
		.HCLK           (I_clk),			// 50M
		.hwRstn         (I_rst_n)
	  );	
	  
//===================================================================
// --- AHB Decoder ---	
	ahb_isp ahb_isp_inst(
		.AHB_HRDATA(AHB1HRDATA),
		.AHB_HREADY(AHB1HREADYOUT),//ready signal, slave to MCU master, 1'b1
		.AHB_HRESP(AHB1HRESP),//respone signal, slave to MCU master
		.AHB_HTRANS(AHB1HTRANS),
		.AHB_HBURST(AHB1HBURST),
		.AHB_HPROT(AHB1HPROT),
		.AHB_HSIZE(AHB1HSIZE),
		.AHB_HWRITE(AHB1HWRITE),
		.AHB_HMASTLOCK(AHB1HMASTLOCK),
		.AHB_HMASTER(AHB1HMASTER),
		.AHB_HADDR(AHB1HADDR),
		.AHB_HWDATA(AHB1HWDATA),
		.AHB_HSEL(AHB1HSEL),
		.AHB_HCLK(AHB1HCLK),
		.AHB_HRESETn(AHB1HRESET),
		
		.isp_reg_rd_en(isp_reg_rd_en),
		.isp_reg_wr_en(isp_reg_wr_en),
		.isp_reg_addr(isp_reg_addr),
		.isp_reg_wr_data(isp_reg_wr_data),
		.isp_reg_rd_data(isp_reg_rd_data),
		.isp_rd_rdy(isp_rd_rdy),
        .isp_vs(csi_vs),
		.isp_disp_mode(isp_disp_mode),
		
		.update_valid(update_valid),
		.cam_awb_en(cam_awb_en),
		.cam_awb_gain_r(cam_awb_gain_r),
		.cam_awb_gain_g(cam_awb_gain_g),
		.cam_awb_gain_b(cam_awb_gain_b),
		.cam_agc_en(cam_agc_en),
		.cam_agc_gain(cam_agc_gain),
		.cam_aec_en(cam_aec_en),
		.cam_aec_exposure(cam_aec_exposure)
	);
	
//assign isp_rd_rdy=1'b1;

endmodule