/*
 ******************************************************************************************
 *
 * 	Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file		gw_isp.c
 * @author		HK Gowin
 * @version		V1.0.0
 * @date		2021-01-01 09:00:00
 * @brief		Gowin ISP MCU Demo
 *
 ******************************************************************************************
 */
 
/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include "gw_isp.h"
#include "gw_isp_param.h"

/* -------------------------- ISP Define ---------------------------- */
const char starts_str[] = "**********************************************************************************\r\n";

const char cmd_help_str[] =
" * \r\n" \
" * Help Info:\r\n" \
" * \r\n" \
" * ISP_CMD Format:		Addr(12bit)	    Value(32bit) #(END)\r\n" \
" * eg,\r\n" \
" * SET REG_CAM_AWB_R: 	   0x104		    0x00000400 #\r\n" \
" * \r\n";

const char init_info_str[] = 
" * \r\n" \
" * Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd.\r\n"
" * \r\n"
" * @Brief: Gowin ISP MCU Demo \r\n"		
" * @Author: HK GOWIN \r\n"
" * @Version: V1.0.0 \r\n"
;
uint32_t Histogram_Buffer[256];
/* -------------------------- ISP Funtion ---------------------------- */
void ISP_help_info_print(void)
{
	printf("\r\n");
	printf(starts_str);
	printf(cmd_help_str);
	printf(starts_str);
	printf("\r\n");
}

void ISP_init_info_print(void)
{
	printf("\r\n");
	printf(starts_str);
	printf(init_info_str);
	printf(" * @Compiled: "__DATE__", "__TIME__"\r\n");
	printf(" * \r\n");
	printf(starts_str);
	printf("\r\n");
	
}

int ISP_cmd_scanf(const uint8_t *str, ISP_t* isp_p)
{
	
	int t;
	uint32_t addr;
	uint32_t value;
	
	t = sscanf((const char *)str,
		" 0x%3X 0x%8X",
		&addr, &value);
		
	if(2 != t)
	{
		return -1;
	}
	
	isp_p->addr = addr;
	isp_p->value = value;
	return 0;
}

void ISP_cmd_print(ISP_t* isp_p)
{

	printf("Set ISP's Reg: 0x%X, 0x%X \r\n", isp_p->addr, isp_p->value);
}

void ISP_init(ISP_t* isp_p)
{
	printf("ISP initializing ... \r\n");
	// --- ISP Value initial
	isp_p->base_addr = AHB_BaseAddr;
	isp_p->img_height = IMG_HEIGHT;
	isp_p->img_width = IMG_WIDTH;
	isp_p->AWB_gain_R = 0x0400;
	isp_p->AWB_gain_B = 0x0400;
	isp_p->AWB_gain_G = 0x0400;
	isp_p->AE_exposure = 0x1FFF;
	isp_p->frame_cnt = 0;
	isp_p->AWB_Process_En = 1;
	isp_p->AE_Process_En = 1;
	
	
	// --- camera param initial
	ISP_SetValue(isp_p, REG_CAM_AWB_EN, CAM_AWB_EN);
	ISP_SetValue(isp_p, REG_CAM_AWB_R, CAM_AWB_R);
	ISP_SetValue(isp_p, REG_CAM_AWB_G, CAM_AWB_G);
	ISP_SetValue(isp_p, REG_CAM_AWB_B, CAM_AWB_B);
	ISP_SetValue(isp_p, REG_CAM_AGC_EN, CAM_AGC_EN);
	ISP_SetValue(isp_p, REG_CAM_AGC_GAIN, CAM_AGC_GAIN);
	ISP_SetValue(isp_p, REG_CAM_AEC_EXPOSURE, CAM_AEC_EXPOSRUE);
	ISP_SetValue(isp_p, REG_CAM_AEC_EN, CAM_AEC_EN);
	
	printf("ISP initial finished! \r\n");
}


void ISP_AE_Process(ISP_t* isp_p)
{
	uint16_t i, sum;
	
	// --- Get value of gain, mean
	printf("AE Processing ...\r\n");
	// --- AE algorithm -----------------------------	

	ISP_SetValue(isp_p, ISP_REG_LOAD, REG_AEAWB_HISTO_RGB_GET);	
	Histogram_Buffer[0] = reg_read(isp_p->base_addr, REG_AEAWB_HISTO_RGB_GET);
	isp_p->Histogram_Buffer_Y[0] = (((Histogram_Buffer[0])&0x07FF) + ((Histogram_Buffer[0]>>10)&0x07FF) + ((Histogram_Buffer[0]>>20)&0x07FF));
	isp_p->AWB_mean_Y = 0;
	sum = 0;
	for (i=1;i<256;i++)
		{
			Histogram_Buffer[i] = reg_read(isp_p->base_addr, REG_AEAWB_HISTO_RGB_GET);
			isp_p->sum_RGB[i] = (((Histogram_Buffer[i])&0x07FF) + ((Histogram_Buffer[i]>>10)&0x07FF) + ((Histogram_Buffer[i]>>20)&0x07FF));
			isp_p->Histogram_Buffer_Y[i] = (isp_p->Histogram_Buffer_Y[i-1]) + (isp_p->sum_RGB[i]);
		}
	for (i=1;i<256;i++)
		{
			if((isp_p->Histogram_Buffer_Y[i])*100/(isp_p->Histogram_Buffer_Y[255])>=95){
				isp_p->AWB_mean_Y = (isp_p->AWB_mean_Y) + i*(isp_p->sum_RGB[i]);
				sum = sum + (isp_p->sum_RGB[i]);
			}
		}	
	isp_p->AWB_mean_Y = (isp_p->AWB_mean_Y) / sum;
	// printf("AWB_mean_Y : %d \r\n", isp_p->AWB_mean_Y);
	if ((isp_p->AWB_mean_Y < 113) && ((isp_p->AE_exposure) <= 0x3FFF)){
		if (isp_p->AWB_mean_Y < 100){
			isp_p->AE_exposure = isp_p->AE_exposure + 500;
		}
		else{
			isp_p->AE_exposure = isp_p->AE_exposure + 50;
		}
		ISP_SetValue(isp_p, REG_CAM_AEC_EXPOSURE, isp_p->AE_exposure);
	}
	else if ((isp_p->AWB_mean_Y > 180) && ((isp_p->AE_exposure) >= 0x0FFF)){
		if (isp_p->AWB_mean_Y > 200){
			isp_p->AE_exposure = isp_p->AE_exposure - 500;
		}
		else{
			isp_p->AE_exposure = isp_p->AE_exposure - 50;
		}
		ISP_SetValue(isp_p, REG_CAM_AEC_EXPOSURE, isp_p->AE_exposure);
	}
	// printf("AE_exposure: %d\r\n", isp_p->AE_exposure); 
}

void ISP_AWB_Process(ISP_t* isp_p)
{
	printf("AWB Processing ...\r\n");
	// --- Get value of gain, mean
	ISP_SetValue(isp_p, ISP_REG_LOAD, REG_AEAWB_MEAN_R);
	isp_p->AWB_mean_R = ISP_GetValue(isp_p, REG_AEAWB_MEAN_R);
	
	ISP_SetValue(isp_p, ISP_REG_LOAD, REG_AEAWB_MEAN_G);
	isp_p->AWB_mean_G = ISP_GetValue(isp_p, REG_AEAWB_MEAN_G);
	
	ISP_SetValue(isp_p, ISP_REG_LOAD, REG_AEAWB_MEAN_B);
	isp_p->AWB_mean_B = ISP_GetValue(isp_p, REG_AEAWB_MEAN_B);
	
	// printf("mean value of R: %d\r\n", isp_p->AWB_mean_R); 
	// printf("mean value of G: %d\r\n", isp_p->AWB_mean_G); 
	// printf("mean value of B: %d\r\n", isp_p->AWB_mean_B); 
	
	
	// --- AWB algorithm -----------------------------		
		if (isp_p->AWB_mean_R >= isp_p->AWB_mean_G){
			if ((isp_p->AWB_mean_R - isp_p->AWB_mean_G) > 250000){
				isp_p->AWB_gain_R = isp_p->AWB_gain_R - 25;  
				ISP_SetValue(isp_p, REG_CAM_AWB_R, isp_p->AWB_gain_R);
				// printf("AWB_gain value of R: %d\r\n", isp_p->AWB_gain_R); 
			}
		}
		else {
				if ((isp_p->AWB_mean_G - isp_p->AWB_mean_R) > 250000){
					isp_p->AWB_gain_R = isp_p->AWB_gain_R + 25;
					ISP_SetValue(isp_p, REG_CAM_AWB_R, isp_p->AWB_gain_R);
					// printf("AWB_gain value of R: %d\r\n", isp_p->AWB_gain_R); 
				}
		}
		if (isp_p->AWB_mean_B > isp_p->AWB_mean_G){
			if ((isp_p->AWB_mean_B - isp_p->AWB_mean_G) > 250000){
				isp_p->AWB_gain_B = isp_p->AWB_gain_B - 25;  
				ISP_SetValue(isp_p, REG_CAM_AWB_B, isp_p->AWB_gain_B);
				// printf("AWB_gain value of B: %d\r\n", isp_p->AWB_gain_B); 
			}
		}
		else { 
			if ((isp_p->AWB_mean_G - isp_p->AWB_mean_B) > 250000){
				isp_p->AWB_gain_B = isp_p->AWB_gain_B + 25;
				ISP_SetValue(isp_p, REG_CAM_AWB_B, isp_p->AWB_gain_B);
				// printf("AWB_gain value of B: %d\r\n", isp_p->AWB_gain_B); 
			}
		}
}

void ISP_MeanDisp(ISP_t* isp_p)
{
	uint32_t mean_R, mean_G, mean_B, mean_Y;

	mean_R = ISP_GetValue(isp_p, REG_AEAWB_MEAN_R);
	mean_G = ISP_GetValue(isp_p, REG_AEAWB_MEAN_G);
	mean_B = ISP_GetValue(isp_p, REG_AEAWB_MEAN_B);
	mean_Y = ISP_GetValue(isp_p, REG_AEAWB_MEAN_Y);
	
	printf("mean value of R: %X\r\n", mean_R); 
	printf("mean value of G: %X\r\n", mean_G); 
	printf("mean value of B: %X\r\n", mean_B); 
	printf("mean value of Y: %X\r\n", mean_Y);

	ISP_SetValue(isp_p, ISP_REG_LOAD, 0x5ff);
	reg_read(isp_p->base_addr, ISP_REG_GET);
	
}

void ISP_HistoDisp(ISP_t* isp_p)
{
	uint32_t isp_histo_ready;
	uint16_t i;
	do{
		isp_histo_ready = ISP_GetValue(isp_p, REG_AEAWB_HISTO_RDY_GET);
		printf("rdy: %X\r\n", isp_histo_ready);
	}while(0x1 != isp_histo_ready );
		
	
	printf("Histogram Value: R G B\r\n");	
	ISP_SetValue(isp_p, ISP_REG_LOAD, REG_AEAWB_HISTO_RGB_GET);	
	for (i=0;i<256;i++)
	{
		Histogram_Buffer[i] = reg_read(isp_p->base_addr, REG_AEAWB_HISTO_RGB_GET);
	}
	for (i=0;i<256;i++)
	{
		printf("%X: %X %X %X \r\n", i, (Histogram_Buffer[i])&0x07FF, (Histogram_Buffer[i]>>10)&0x07FF, (Histogram_Buffer[i]>>20)&0x07FF); // R G B with 10 bits

	}
	ISP_SetValue(isp_p, ISP_REG_LOAD, 0x5ff);
	reg_read(isp_p->base_addr, ISP_REG_GET);
	
}



void ISP_SetValue(ISP_t* isp_p, uint32_t addr, uint32_t value)
{
	reg_write(isp_p->base_addr, addr, value);
}

uint32_t ISP_GetValue(ISP_t* isp_p, uint32_t addr)
{
	reg_write(isp_p->base_addr, addr, 0);
	return reg_read(isp_p->base_addr, addr);
}

//delay
void delay(__IO uint32_t nCount)
{
	for(; nCount != 0; nCount--);
}
