/*
 ******************************************************************************************
 *
 * 	Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file		gw_isp.h
 * @author		HK Gowin
 * @version		V1.0.0
 * @date		2021-01-01 09:00:00
 * @brief		Gowin ISP MCU Demo
 *
 ******************************************************************************************
 */

#ifndef GW_ISP_H
#define GW_ISP_H

/* Includes ---------------------------------------------------------*/
#include "GOWIN_M1.h"
#include "gw_ahb.h"

/* -------------------------- ISP Define---------------------------- */
// --- ISP Reg ---
#define ISP_CMD 						0x000
#define ISP_REG_LOAD 					0x004
#define ISP_REG_GET						0x008
#define ISP_IMAGE_RD					0x00C
	
// --- Sensor Reg ---			
#define REG_CAM_AWB_EN 					0x100
#define REG_CAM_AWB_R 					0x104
#define REG_CAM_AWB_G 					0x108
#define REG_CAM_AWB_B 					0x10C
#define REG_CAM_AGC_EN 					0x110
#define REG_CAM_AGC_GAIN 				0x114
#define REG_CAM_AEC_EN 					0x118
#define REG_CAM_AEC_EXPOSURE 			0x11C
// --- CCM Reg ---			
#define REG_CCM_K11 					0x200
#define REG_CCM_K12 					0x204
#define REG_CCM_K13 					0x208
#define REG_CCM_K21 					0x20C
#define REG_CCM_K22 					0x210
#define REG_CCM_K23 					0x214
#define REG_CCM_K31 					0x218
#define REG_CCM_K32 					0x21C
#define REG_CCM_K33 					0x220
#define REG_CCM_O1 						0x224
#define REG_CCM_O2 						0x228
#define REG_CCM_O3 						0x22C

#define REG_AEAWB_GET_AWB_GAIN_R 		0x410
#define REG_AEAWB_GET_AWB_GAIN_G 		0x414
#define REG_AEAWB_GET_AWB_GAIN_B 		0x418
#define REG_AEAWB_GET_AE_GAIN 			0x41C
#define REG_AEAWB_MEAN_R 				0x420
#define REG_AEAWB_MEAN_G 				0x424
#define REG_AEAWB_MEAN_B 				0x428
#define REG_AEAWB_MEAN_Y 				0x42C
#define REG_AEAWB_HISTO_RDY_GET  		0x460
#define REG_AEAWB_HISTO_RGB_GET  		0x500




#pragma pack(push, 1)			//自定义结构体中的成员为按1字节对齐
typedef struct{
	uint32_t 	base_addr;
    uint32_t 	addr;
    uint32_t 	value;
	uint8_t 	state;
	uint16_t 	img_height;
	uint16_t 	img_width;
	uint16_t	frame_cnt;
	uint32_t	AWB_gain_R;
	uint32_t	AWB_gain_G;
	uint32_t	AWB_gain_B;
	uint32_t	AWB_mean_R;
	uint32_t	AWB_mean_G;
	uint32_t	AWB_mean_B;
	uint32_t	AWB_mean_Y;
	uint32_t	AE_exposure;
	uint32_t    sum_RGB[256];
	uint32_t    Histogram_Buffer_Y[256];
	bool 		AWB_Process_En;
	bool		AE_Process_En;
	
}ISP_t;
#pragma pack(pop)				//取消自定义字节对齐方式



/* -------------------------- ISP function---------------------------- */
void ISP_init_info_print(void);
void ISP_help_info_print(void);

int ISP_cmd_scanf(const uint8_t *str, ISP_t* isp_p);
void ISP_cmd_print(ISP_t* isp_p);

void ISP_init(ISP_t* isp_p);
void ISP_img_capture(ISP_t* isp_p);

void ISP_AE_Process(ISP_t* isp_p);
void ISP_AWB_Process(ISP_t* isp_p);


void ISP_MeanDisp(ISP_t* isp_p);
void ISP_HistoDisp(ISP_t* isp_p);

void ISP_SetValue(ISP_t* isp_p, uint32_t addr, uint32_t value);
uint32_t ISP_GetValue(ISP_t* isp_p, uint32_t addr);

void delay(__IO uint32_t nCount);


#endif
