/*
 ******************************************************************************************
 *
 * 	Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file		main.c
 * @author		HK Gowin
 * @version		V1.0.0
 * @date		2021-01-01 09:00:00
 * @brief		Gowin ISP MCU Demo
 *
 ******************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include "main.h"

ISP_t isp_p;

int main()
{
	SystemInit();		// system init
	GPIOInit();			// GPIO init
	UartInit();			// uart init
	
	ISP_init_info_print();
	ISP_init(&isp_p);
	ISP_help_info_print();
	
	NVIC_EnableIRQ(GPIO0_0_IRQn); 
    NVIC_SetPriority(GPIO0_0_IRQn,0);
	NVIC_EnableIRQ(UART0_IRQn);				//Enable uart0 interrupt
	while(1);
}

static void GPIOInit(void)
{
    GPIO_InitTypeDef    GPIO_InitStruct;

    //GPIO中断输入
    GPIO_InitStruct.GPIO_Pin  = GPIO_Pin_0;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_IN; 
    GPIO_InitStruct.GPIO_Int  = GPIO_Int_Rising_Edge; 
    
    GPIO_Init(GPIO0,&GPIO_InitStruct);    

}    

static void UartInit(void)
{
  UART_InitTypeDef UART_InitStruct;
	
  UART_InitStruct.UART_Mode.UARTMode_Tx = ENABLE;
  UART_InitStruct.UART_Mode.UARTMode_Rx = ENABLE;
  UART_InitStruct.UART_Int.UARTInt_Tx = DISABLE;
  UART_InitStruct.UART_Int.UARTInt_Rx = ENABLE;
  UART_InitStruct.UART_Ovr.UARTOvr_Tx = DISABLE;
  UART_InitStruct.UART_Ovr.UARTOvr_Rx = DISABLE;
  UART_InitStruct.UART_Hstm = DISABLE;
  UART_InitStruct.UART_BaudRate = 115200;//Baud Rate
	
  UART_Init(UART0,&UART_InitStruct);
}

/**
  * @brief  This function handles GPIO0_0 interrupt request.
  * @param  none
  * @retval none
  */
void GPIO0_0_Handler(void)
{

	if(isp_p.frame_cnt == 5)
	{
		if(0x1 == isp_p.AE_Process_En)
		{
			ISP_AE_Process(&isp_p);
		}
		
		if(0x1 == isp_p.AWB_Process_En)
		{
			ISP_AWB_Process(&isp_p);
		}
	}
	else 
	{
		isp_p.frame_cnt = isp_p.frame_cnt + 1;
	}
	ISP_SetValue(&isp_p, ISP_REG_LOAD, 0x5ff);
	reg_read(isp_p.base_addr, ISP_REG_GET);
	GPIO_IntClear(GPIO0,GPIO_Pin_0);
}

/**
  * @brief  This function handles UART0 interrupt request.
  * @param  none
  * @retval none
  */
void UART0_Handler(void)
{
	uint16_t len = 0;
	uint8_t flag_cmd_char_end = 0;
	
	while (len < CMD_STR_LEN_MAX)
	{
		if(UART_GetRxIRQStatus(UART0) == SET)
		{
			cmd_str[len] = UART_ReceiveChar(UART0);
			
			if (('\r' == cmd_str[len])
					|| ('\t' == cmd_str[len]))
			{
				continue;
			}
		
			if ('#' == cmd_str[len])
			{
				cmd_str[len] = '\0';
				len++;
				flag_cmd_char_end = 1;
				if(0 == ISP_cmd_scanf(cmd_str, &isp_p))
				{
					ISP_cmd_print(&isp_p);
					if(0x0 == isp_p.addr && 0x1 == isp_p.value)
						ISP_MeanDisp(&isp_p);
					else if(0x0 == isp_p.addr && 0x2 == isp_p.value)
						ISP_HistoDisp(&isp_p);
					else if(0x0 == isp_p.addr && 0x3 == isp_p.value)
						isp_p.AWB_Process_En = !isp_p.AWB_Process_En;
					else if(0x0 == isp_p.addr && 0x4 == isp_p.value)
						isp_p.AE_Process_En = !isp_p.AE_Process_En;
					else if(0x44C == isp_p.addr){
						isp_p.AE_exposure = isp_p.value;
						ISP_SetValue(&isp_p, REG_CAM_AEC_EXPOSURE, isp_p.AE_exposure);
					}
					else
						ISP_SetValue(&isp_p, isp_p.addr, isp_p.value);
				}
				
				UART_ClearRxIRQ(UART0);
				break;
			}
			
			len++;
		}
	}
	
	if (1 != flag_cmd_char_end)
	{
		printf("Error: len or cmd format error\r\n");
	}
	
}



